@@ -44,13 +44,35 @@ void free_layer_configuration(const struct loader_instance* inst, loader_setting
     memset(layer_configuration, 0, sizeof(loader_settings_layer_configuration));
 }
 
+void free_driver_configuration(const struct loader_instance* inst, loader_settings_driver_configuration* driver_configuration) {
+    loader_instance_heap_free(inst, driver_configuration->path);
+    memset(driver_configuration, 0, sizeof(loader_settings_driver_configuration));
+}
+
+void free_device_configuration(const struct loader_instance* inst, loader_settings_device_configuration* device_configuration) {
+    (void)inst;
+    memset(device_configuration, 0, sizeof(loader_settings_device_configuration));
+}
+
 void free_loader_settings(const struct loader_instance* inst, loader_settings* settings) {
     if (NULL != settings->layer_configurations) {
         for (uint32_t i = 0; i < settings->layer_configuration_count; i++) {
             free_layer_configuration(inst, &settings->layer_configurations[i]);
         }
+        loader_instance_heap_free(inst, settings->layer_configurations);
+    }
+    if (NULL != settings->additional_drivers) {
+        for (uint32_t i = 0; i < settings->additional_driver_count; i++) {
+            free_driver_configuration(inst, &settings->additional_drivers[i]);
+        }
+        loader_instance_heap_free(inst, settings->additional_drivers);
+    }
+    if (NULL != settings->device_configurations) {
+        for (uint32_t i = 0; i < settings->device_configuration_count; i++) {
+            free_device_configuration(inst, &settings->device_configurations[i]);
+        }
+        loader_instance_heap_free(inst, settings->device_configurations);
     }
-    loader_instance_heap_free(inst, settings->layer_configurations);
     loader_instance_heap_free(inst, settings->settings_file_path);
     memset(settings, 0, sizeof(loader_settings));
 }
@@ -168,6 +190,7 @@ VkResult parse_layer_configurations(const struct loader_instance* inst, cJSON* s
 
     uint32_t layer_configurations_count = loader_cJSON_GetArraySize(layer_configurations);
     if (layer_configurations_count == 0) {
+        loader_settings->layer_configurations_active = true;
         return VK_SUCCESS;
     }
 
@@ -192,6 +215,7 @@ VkResult parse_layer_configurations(const struct loader_instance* inst, cJSON* s
             goto out;
         }
     }
+    loader_settings->layer_configurations_active = true;
 out:
     if (res != VK_SUCCESS) {
         if (loader_settings->layer_configurations) {
@@ -207,44 +231,352 @@ out:
     return res;
 }
 
-VkResult check_if_settings_path_exists(const struct loader_instance* inst, char* base, char* suffix, char** settings_file_path) {
-    if (NULL == base || NULL == suffix) {
-        return VK_ERROR_INITIALIZATION_FAILED;
+VkResult parse_additional_driver(const struct loader_instance* inst, cJSON* additional_driver_json,
+                                 loader_settings_driver_configuration* additional_driver) {
+    VkResult res = VK_SUCCESS;
+    res = loader_parse_json_string(additional_driver_json, "path", &(additional_driver->path));
+    if (res != VK_SUCCESS) {
+        goto out;
     }
-    size_t base_len = strlen(base);
-    size_t suffix_len = strlen(suffix);
-    size_t path_len = base_len + suffix_len + 1;
-    *settings_file_path = loader_instance_heap_calloc(inst, path_len, VK_SYSTEM_ALLOCATION_SCOPE_COMMAND);
-    if (NULL == *settings_file_path) {
-        return VK_ERROR_OUT_OF_HOST_MEMORY;
+out:
+    if (res != VK_SUCCESS) {
+        free_driver_configuration(inst, additional_driver);
     }
-    loader_strncpy(*settings_file_path, path_len, base, base_len);
-    loader_strncat(*settings_file_path, path_len, suffix, suffix_len);
+    return res;
+}
 
-    if (!loader_platform_file_exists(*settings_file_path)) {
-        loader_instance_heap_free(inst, *settings_file_path);
-        *settings_file_path = NULL;
+VkResult parse_additional_drivers(const struct loader_instance* inst, cJSON* settings_object, loader_settings* loader_settings) {
+    VkResult res = VK_SUCCESS;
+
+    cJSON* additional_drivers_use_exclusively_json =
+        loader_cJSON_GetObjectItem(settings_object, "additional_drivers_use_exclusively");
+    if (additional_drivers_use_exclusively_json && additional_drivers_use_exclusively_json->type == cJSON_True) {
+        loader_settings->additional_drivers_use_exclusively = true;
+    }
+
+    cJSON* additional_drivers_json = loader_cJSON_GetObjectItem(settings_object, "additional_drivers");
+    if (NULL == additional_drivers_json) {
+        return VK_SUCCESS;
+    }
+
+    uint32_t additional_driver_count = loader_cJSON_GetArraySize(additional_drivers_json);
+    if (additional_driver_count == 0) {
+        return VK_SUCCESS;
+    }
+
+    loader_settings->additional_driver_count = additional_driver_count;
+
+    loader_settings->additional_drivers = loader_instance_heap_calloc(
+        inst, sizeof(loader_settings_layer_configuration) * additional_driver_count, VK_SYSTEM_ALLOCATION_SCOPE_INSTANCE);
+    if (NULL == loader_settings->additional_drivers) {
+        res = VK_ERROR_OUT_OF_HOST_MEMORY;
+        goto out;
+    }
+
+    cJSON* driver = NULL;
+    size_t i = 0;
+    cJSON_ArrayForEach(driver, additional_drivers_json) {
+        if (driver->type != cJSON_Object) {
+            res = VK_ERROR_INITIALIZATION_FAILED;
+            goto out;
+        }
+        res = parse_additional_driver(inst, driver, &(loader_settings->additional_drivers[i++]));
+        if (VK_SUCCESS != res) {
+            goto out;
+        }
+    }
+out:
+    if (res != VK_SUCCESS) {
+        if (loader_settings->additional_drivers) {
+            for (size_t index = 0; index < loader_settings->additional_driver_count; index++) {
+                free_driver_configuration(inst, &(loader_settings->additional_drivers[index]));
+            }
+            loader_settings->additional_driver_count = 0;
+            loader_instance_heap_free(inst, loader_settings->additional_drivers);
+            loader_settings->additional_drivers = NULL;
+        }
+    }
+    return res;
+}
+
+VkResult parse_uuid_array(cJSON* device_configuration_json, const char* uuid_name, uint8_t uuid[16]) {
+    cJSON* uuid_array = loader_cJSON_GetObjectItem(device_configuration_json, uuid_name);
+    if (NULL == uuid_array) {
         return VK_ERROR_INITIALIZATION_FAILED;
     }
+
+    if (uuid_array->type != cJSON_Array) {
+        return VK_ERROR_INITIALIZATION_FAILED;
+    }
+    if (VK_UUID_SIZE != loader_cJSON_GetArraySize(uuid_array)) {
+        return VK_ERROR_INITIALIZATION_FAILED;
+    }
+
+    cJSON* uuid_field = NULL;
+    size_t i = 0;
+    cJSON_ArrayForEach(uuid_field, uuid_array) {
+        if (i >= VK_UUID_SIZE) {
+            break;
+        }
+        if (uuid_field->type != cJSON_Number) {
+            return VK_ERROR_INITIALIZATION_FAILED;
+        }
+        if (uuid_field->valueint < 0 || uuid_field->valueint > 255) {
+            return VK_ERROR_INITIALIZATION_FAILED;
+        }
+
+        uuid[i] = (uint8_t)uuid_field->valueint;
+        i++;
+    }
     return VK_SUCCESS;
 }
+
+VkResult parse_device_configuration(const struct loader_instance* inst, cJSON* device_configuration_json,
+                                    loader_settings_device_configuration* device_configuration) {
+    (void)inst;
+    VkResult res = VK_SUCCESS;
+
+    res = parse_uuid_array(device_configuration_json, "deviceUUID", device_configuration->deviceUUID);
+    if (VK_SUCCESS != res) {
+        goto out;
+    }
+
+    res = parse_uuid_array(device_configuration_json, "driverUUID", device_configuration->driverUUID);
+    if (VK_SUCCESS != res) {
+        goto out;
+    }
+
+    cJSON* driverVersion_json = loader_cJSON_GetObjectItem(device_configuration_json, "driverVersion");
+    if (NULL == driverVersion_json || driverVersion_json->type != cJSON_Number) {
+        return VK_ERROR_INITIALIZATION_FAILED;
+    }
+    device_configuration->driverVersion = driverVersion_json->valueint;
+
+    VkResult deviceNameRes = loader_parse_json_string_to_existing_str(
+        device_configuration_json, "deviceName", VK_MAX_PHYSICAL_DEVICE_NAME_SIZE, device_configuration->deviceName);
+    if (VK_ERROR_OUT_OF_HOST_MEMORY == deviceNameRes) {
+        res = deviceNameRes;
+        goto out;
+    }
+
+    VkResult driverNameRes = loader_parse_json_string_to_existing_str(
+        device_configuration_json, "driverName", VK_MAX_PHYSICAL_DEVICE_NAME_SIZE, device_configuration->driverName);
+    if (VK_ERROR_OUT_OF_HOST_MEMORY == driverNameRes) {
+        res = driverNameRes;
+        goto out;
+    }
+out:
+    if (res != VK_SUCCESS) {
+        memset(device_configuration, 0, sizeof(loader_settings_device_configuration));
+    }
+    return res;
+}
+
+VkResult parse_device_configurations(const struct loader_instance* inst, cJSON* settings_object, loader_settings* loader_settings) {
+    VkResult res = VK_SUCCESS;
+
+    cJSON* device_configurations = loader_cJSON_GetObjectItem(settings_object, "device_configurations");
+    if (NULL == device_configurations) {
+        return VK_SUCCESS;
+    }
+
+    uint32_t device_configuration_count = loader_cJSON_GetArraySize(device_configurations);
+    if (device_configuration_count == 0) {
+        loader_settings->device_configurations_active = true;
+        return VK_SUCCESS;
+    }
+
+    loader_settings->device_configuration_count = device_configuration_count;
+
+    loader_settings->device_configurations = loader_instance_heap_calloc(
+        inst, sizeof(loader_settings_device_configuration) * device_configuration_count, VK_SYSTEM_ALLOCATION_SCOPE_INSTANCE);
+    if (NULL == loader_settings->device_configurations) {
+        res = VK_ERROR_OUT_OF_HOST_MEMORY;
+        goto out;
+    }
+
+    cJSON* device = NULL;
+    size_t i = 0;
+    cJSON_ArrayForEach(device, device_configurations) {
+        if (device->type != cJSON_Object) {
+            res = VK_ERROR_INITIALIZATION_FAILED;
+            goto out;
+        }
+        res = parse_device_configuration(inst, device, &(loader_settings->device_configurations[i]));
+        if (res == VK_ERROR_OUT_OF_HOST_MEMORY) {
+            goto out;
+        } else if (res != VK_SUCCESS) {
+            continue;
+        }
+        i++;
+    }
+    loader_settings->device_configurations_active = true;
+out:
+    if (res != VK_SUCCESS) {
+        if (loader_settings->device_configurations) {
+            for (size_t index = 0; index < loader_settings->device_configuration_count; index++) {
+                free_device_configuration(inst, &(loader_settings->device_configurations[index]));
+            }
+            loader_settings->device_configuration_count = 0;
+            loader_instance_heap_free(inst, loader_settings->device_configurations);
+            loader_settings->device_configurations = NULL;
+        }
+    }
+    return res;
+}
+
+#if COMMON_UNIX_PLATFORMS
+// Given a base and suffix path, determine if a file at that location exists, and if it is return success.
+// Since base may contain multiple paths separated by PATH_SEPARATOR, we must extract each segment and check segment + suffix
+// individually
+VkResult check_if_settings_path_exists(const struct loader_instance* inst, const char* base, const char* suffix,
+                                       char** settings_file_path) {
+    if (NULL == base || NULL == suffix) {
+        return VK_ERROR_INITIALIZATION_FAILED;
+    }
+
+    size_t base_len = strlen(base);
+    size_t suffix_len = strlen(suffix);
+
+    uint32_t start = 0;
+    uint32_t stop = 0;
+    while (base[start] != '\0' && start < base_len && stop < base_len) {
+        start = stop;
+        stop = start + 1;
+        while (base[stop] != PATH_SEPARATOR && base[stop] != '\0' && stop < base_len) {
+            stop++;
+        }
+
+        size_t segment_len = (stop - start);
+        if (segment_len <= 1) {
+            // segment is *just* a PATH_SEPARATOR, skip it
+            continue;
+        }
+        size_t path_len = segment_len + suffix_len + 1;
+        *settings_file_path = loader_instance_heap_calloc(inst, path_len, VK_SYSTEM_ALLOCATION_SCOPE_COMMAND);
+        if (NULL == *settings_file_path) {
+            return VK_ERROR_OUT_OF_HOST_MEMORY;
+        }
+        loader_strncpy(*settings_file_path, path_len, base + start, segment_len);
+        loader_strncat(*settings_file_path, path_len, suffix, suffix_len);
+
+        if (loader_platform_file_exists(*settings_file_path)) {
+            return VK_SUCCESS;
+        }
+        loader_instance_heap_free(inst, *settings_file_path);
+        *settings_file_path = NULL;
+    }
+
+    return VK_ERROR_INITIALIZATION_FAILED;
+}
+
+// Follow the logic of read_data_files_in_search_paths but only look for "/vulkan/loader_settings.d/" VK_LOADER_SETTINGS_FILENAME
 VkResult get_unix_settings_path(const struct loader_instance* inst, char** settings_file_path) {
-    VkResult res =
-        check_if_settings_path_exists(inst, loader_secure_getenv("HOME", inst),
-                                      "/.local/share/vulkan/loader_settings.d/" VK_LOADER_SETTINGS_FILENAME, settings_file_path);
+    // First, get XDG env-vars we use. Don't need to worry about free'ing because on linux getenv is non-allocating
+    char* xdg_config_home = loader_secure_getenv("XDG_CONFIG_HOME", inst);
+    char* xdg_config_dirs = loader_secure_getenv("XDG_CONFIG_DIRS", inst);
+    char* xdg_data_home = loader_secure_getenv("XDG_DATA_HOME", inst);
+    char* xdg_data_dirs = loader_secure_getenv("XDG_DATA_DIRS", inst);
+
+    // Use fallback directories for xdg_config_dirs and xdg_data_dirs if they are NULL.
+#if !defined(__Fuchsia__) && !defined(__QNX__)
+    if (NULL == xdg_config_dirs || '\0' == xdg_config_dirs[0]) {
+        xdg_config_dirs = FALLBACK_CONFIG_DIRS;
+    }
+#endif
+
+#if !defined(__Fuchsia__) && !defined(__QNX__)
+    if (NULL == xdg_data_dirs || '\0' == xdg_data_dirs[0]) {
+        xdg_data_dirs = FALLBACK_DATA_DIRS;
+    }
+#endif
+
+    VkResult res = check_if_settings_path_exists(
+        inst, xdg_config_home, "/" VULKAN_DIR "/loader_settings.d/" VK_LOADER_SETTINGS_FILENAME, settings_file_path);
     if (res == VK_SUCCESS) {
         return res;
     }
-    // If HOME isn't set, fallback to XDG_DATA_HOME
-    res = check_if_settings_path_exists(inst, loader_secure_getenv("XDG_DATA_HOME", inst),
-                                        "/vulkan/loader_settings.d/" VK_LOADER_SETTINGS_FILENAME, settings_file_path);
+
+    res = check_if_settings_path_exists(inst, xdg_data_home, "/" VULKAN_DIR "/loader_settings.d/" VK_LOADER_SETTINGS_FILENAME,
+                                        settings_file_path);
     if (res == VK_SUCCESS) {
         return res;
     }
-    // if XDG_DATA_HOME isn't set, fallback to /etc.
-    // note that the settings_fil_path_suffix stays the same since its the same layout as for XDG_DATA_HOME
-    return check_if_settings_path_exists(inst, "/etc", "/vulkan/loader_settings.d/" VK_LOADER_SETTINGS_FILENAME,
-                                         settings_file_path);
+
+    // Check home if either xdg_config_home or xdg_data_home wasn't set
+    char* home = loader_secure_getenv("HOME", inst);
+    if (home != NULL) {
+        if (NULL == xdg_config_home || '\0' == xdg_config_home[0]) {
+            res = check_if_settings_path_exists(
+                inst, home, "/.config/" VULKAN_DIR "/loader_settings.d/" VK_LOADER_SETTINGS_FILENAME, settings_file_path);
+            if (res == VK_SUCCESS) {
+                return res;
+            }
+        }
+        if (NULL == xdg_data_home || '\0' == xdg_data_home[0]) {
+            res = check_if_settings_path_exists(
+                inst, home, "/.local/share/" VULKAN_DIR "/loader_settings.d/" VK_LOADER_SETTINGS_FILENAME, settings_file_path);
+            if (res == VK_SUCCESS) {
+                return res;
+            }
+        }
+    }
+
+    res = check_if_settings_path_exists(inst, xdg_config_dirs, "/" VULKAN_DIR "/loader_settings.d/" VK_LOADER_SETTINGS_FILENAME,
+                                        settings_file_path);
+    if (res == VK_SUCCESS) {
+        return res;
+    }
+
+    res = check_if_settings_path_exists(inst, SYSCONFDIR, "/" VULKAN_DIR "/loader_settings.d/" VK_LOADER_SETTINGS_FILENAME,
+                                        settings_file_path);
+    if (res == VK_SUCCESS) {
+        return res;
+    }
+#if defined(EXTRASYSCONFDIR)
+
+    res = check_if_settings_path_exists(inst, EXTRASYSCONFDIR, "/" VULKAN_DIR "/loader_settings.d/" VK_LOADER_SETTINGS_FILENAME,
+                                        settings_file_path);
+    if (res == VK_SUCCESS) {
+        return res;
+    }
+#endif
+    res = check_if_settings_path_exists(inst, xdg_data_dirs, "/" VULKAN_DIR "/loader_settings.d/" VK_LOADER_SETTINGS_FILENAME,
+                                        settings_file_path);
+    if (res == VK_SUCCESS) {
+        return res;
+    }
+
+    return VK_ERROR_INITIALIZATION_FAILED;
+}
+#endif
+
+bool check_if_layer_configurations_are_equal(loader_settings_layer_configuration* a, loader_settings_layer_configuration* b) {
+    if (!a->name || !b->name || 0 != strcmp(a->name, b->name)) {
+        return false;
+    }
+    if (!a->path || !b->path || 0 != strcmp(a->path, b->path)) {
+        return false;
+    }
+    return a->control == b->control;
+}
+
+bool check_if_driver_configurations_are_equal(loader_settings_driver_configuration* a, loader_settings_driver_configuration* b) {
+    if (!a->path || !b->path || 0 != strcmp(a->path, b->path)) {
+        return false;
+    }
+    return true;
+}
+
+bool check_if_device_configurations_are_equal(loader_settings_device_configuration* a, loader_settings_device_configuration* b) {
+    for (uint32_t i = 0; i < VK_UUID_SIZE; i++) {
+        if (a->deviceUUID[i] != b->deviceUUID[i]) return false;
+    }
+    for (uint32_t i = 0; i < VK_UUID_SIZE; i++) {
+        if (a->driverUUID[i] != b->driverUUID[i]) return false;
+    }
+    if (a->driverVersion != b->driverVersion) return false;
+    return true;
 }
 
 bool check_if_settings_are_equal(loader_settings* a, loader_settings* b) {
@@ -254,20 +586,20 @@ bool check_if_settings_are_equal(loader_settings* a, loader_settings* b) {
     are_equal &= a->settings_active == b->settings_active;
     are_equal &= a->has_unordered_layer_location == b->has_unordered_layer_location;
     are_equal &= a->debug_level == b->debug_level;
+    are_equal &= a->layer_configurations_active == b->layer_configurations_active;
     are_equal &= a->layer_configuration_count == b->layer_configuration_count;
+    are_equal &= a->additional_driver_count == b->additional_driver_count;
+    are_equal &= a->device_configurations_active == b->device_configurations_active;
+    are_equal &= a->device_configuration_count == b->device_configuration_count;
     if (!are_equal) return false;
     for (uint32_t i = 0; i < a->layer_configuration_count && i < b->layer_configuration_count; i++) {
-        if (a->layer_configurations[i].name && b->layer_configurations[i].name) {
-            are_equal &= 0 == strcmp(a->layer_configurations[i].name, b->layer_configurations[i].name);
-        } else {
-            are_equal = false;
-        }
-        if (a->layer_configurations[i].path && b->layer_configurations[i].path) {
-            are_equal &= 0 == strcmp(a->layer_configurations[i].path, b->layer_configurations[i].path);
-        } else {
-            are_equal = false;
-        }
-        are_equal &= a->layer_configurations[i].control == b->layer_configurations[i].control;
+        are_equal &= check_if_layer_configurations_are_equal(&a->layer_configurations[i], &b->layer_configurations[i]);
+    }
+    for (uint32_t i = 0; i < a->additional_driver_count && i < b->additional_driver_count; i++) {
+        are_equal &= check_if_driver_configurations_are_equal(&a->additional_drivers[i], &b->additional_drivers[i]);
+    }
+    for (uint32_t i = 0; i < a->device_configuration_count && i < b->device_configuration_count; i++) {
+        are_equal &= check_if_device_configurations_are_equal(&a->device_configurations[i], &b->device_configurations[i]);
     }
     return are_equal;
 }
@@ -288,18 +620,50 @@ void log_settings(const struct loader_instance* inst, loader_settings* settings)
     if (strlen(cmd_line_msg)) {
         loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "Loader Settings Filters for Logging to Standard Error: %s", cmd_line_msg);
     }
-
-    loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "Layer Configurations count = %d", settings->layer_configuration_count);
-    for (uint32_t i = 0; i < settings->layer_configuration_count; i++) {
-        loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "---- Layer Configuration [%d] ----", i);
-        if (settings->layer_configurations[i].control != LOADER_SETTINGS_LAYER_UNORDERED_LAYER_LOCATION) {
-            loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "Name: %s", settings->layer_configurations[i].name);
-            loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "Path: %s", settings->layer_configurations[i].path);
-            loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "Layer Type: %s",
-                       settings->layer_configurations[i].treat_as_implicit_manifest ? "Implicit" : "Explicit");
+    if (settings->layer_configurations_active) {
+        loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "Layer Configurations count = %d", settings->layer_configuration_count);
+        for (uint32_t i = 0; i < settings->layer_configuration_count; i++) {
+            loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "---- Layer Configuration [%d] ----", i);
+            if (settings->layer_configurations[i].control != LOADER_SETTINGS_LAYER_UNORDERED_LAYER_LOCATION) {
+                loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "Name: %s", settings->layer_configurations[i].name);
+                loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "Path: %s", settings->layer_configurations[i].path);
+                loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "Layer Type: %s",
+                           settings->layer_configurations[i].treat_as_implicit_manifest ? "Implicit" : "Explicit");
+            }
+            loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "Control: %s",
+                       loader_settings_layer_control_to_string(settings->layer_configurations[i].control));
+        }
+    }
+    if (settings->additional_driver_count > 0) {
+        loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "----");
+        loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "Use Additional Drivers Exclusively = %s",
+                   settings->additional_drivers_use_exclusively ? "true" : "false");
+        loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "Additional Driver Configurations count = %d",
+                   settings->additional_driver_count);
+        for (uint32_t i = 0; i < settings->additional_driver_count; i++) {
+            loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "---- Driver Configuration [%d] ----", i);
+            loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "Path: %s", settings->additional_drivers[i].path);
+        }
+    }
+    if (settings->device_configurations_active) {
+        loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "----");
+        loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "Device Configurations count = %d", settings->device_configuration_count);
+        for (uint32_t i = 0; i < settings->device_configuration_count; i++) {
+            loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "---- Device Configuration [%d] ----", i);
+            char device_uuid_str[UUID_STR_LEN] = {0};
+            loader_log_generate_uuid_string(settings->device_configurations[i].deviceUUID, device_uuid_str);
+            char driver_uuid_str[UUID_STR_LEN] = {0};
+            loader_log_generate_uuid_string(settings->device_configurations[i].driverUUID, driver_uuid_str);
+            loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "deviceUUID: %s", device_uuid_str);
+            loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "driverUUID: %s", driver_uuid_str);
+            loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "driverVersion: %d", settings->device_configurations[i].driverVersion);
+            if ('\0' != settings->device_configurations[i].deviceName[0]) {
+                loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "deviceName: %s", settings->device_configurations[i].deviceName);
+            }
+            if ('\0' != settings->device_configurations[i].driverName[0]) {
+                loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "driverName: %s", settings->device_configurations[i].driverName);
+            }
         }
-        loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "Control: %s",
-                   loader_settings_layer_control_to_string(settings->layer_configurations[i].control));
     }
     loader_log(inst, VULKAN_LOADER_DEBUG_BIT, 0, "---------------------------------");
 }
@@ -427,8 +791,10 @@ VkResult get_loader_settings(const struct loader_instance* inst, loader_settings
     cJSON* stderr_filter = loader_cJSON_GetObjectItem(settings_to_use, "stderr_log");
     if (NULL != stderr_filter) {
         struct loader_string_list stderr_log = {0};
-        res = loader_parse_json_array_of_strings(inst, settings_to_use, "stderr_log", &stderr_log);
-        if (VK_ERROR_OUT_OF_HOST_MEMORY == res) {
+        VkResult stderr_log_result = VK_SUCCESS;
+        stderr_log_result = loader_parse_json_array_of_strings(inst, settings_to_use, "stderr_log", &stderr_log);
+        if (VK_ERROR_OUT_OF_HOST_MEMORY == stderr_log_result) {
+            res = VK_ERROR_OUT_OF_HOST_MEMORY;
             goto out;
         }
         loader_settings->debug_level = parse_log_filters_from_strings(&stderr_log);
@@ -442,22 +808,23 @@ VkResult get_loader_settings(const struct loader_instance* inst, loader_settings
         cJSON_ArrayForEach(log_element, logs_to_use) {
             // bool is_valid = true;
             struct loader_string_list log_destinations = {0};
-            res = loader_parse_json_array_of_strings(inst, log_element, "destinations", &log_destinations);
-            if (res != VK_SUCCESS) {
+            VkResult parse_dest_res = loader_parse_json_array_of_strings(inst, log_element, "destinations", &log_destinations);
+            if (parse_dest_res != VK_SUCCESS) {
                 // is_valid = false;
             }
             free_string_list(inst, &log_destinations);
             struct loader_string_list log_filters = {0};
-            res = loader_parse_json_array_of_strings(inst, log_element, "filters", &log_filters);
-            if (res != VK_SUCCESS) {
+            VkResult parse_filters_res = loader_parse_json_array_of_strings(inst, log_element, "filters", &log_filters);
+            if (parse_filters_res != VK_SUCCESS) {
                 // is_valid = false;
             }
             free_string_list(inst, &log_filters);
         }
     }
 
-    res = parse_layer_configurations(inst, settings_to_use, loader_settings);
-    if (res != VK_SUCCESS) {
+    VkResult layer_configurations_res = parse_layer_configurations(inst, settings_to_use, loader_settings);
+    if (VK_ERROR_OUT_OF_HOST_MEMORY == layer_configurations_res) {
+        res = layer_configurations_res;
         goto out;
     }
 
@@ -470,9 +837,29 @@ VkResult get_loader_settings(const struct loader_instance* inst, loader_settings
         }
     }
 
-    loader_settings->settings_file_path = settings_file_path;
-    settings_file_path = NULL;
-    loader_settings->settings_active = true;
+    VkResult additional_drivers_res = parse_additional_drivers(inst, settings_to_use, loader_settings);
+    if (VK_ERROR_OUT_OF_HOST_MEMORY == additional_drivers_res) {
+        res = additional_drivers_res;
+        goto out;
+    }
+
+    VkResult device_configurations_res = parse_device_configurations(inst, settings_to_use, loader_settings);
+    if (VK_ERROR_OUT_OF_HOST_MEMORY == device_configurations_res) {
+        res = device_configurations_res;
+        goto out;
+    }
+
+    // Only consider the settings active if there is at least one "setting" active.
+    // Those are either logging, layers, additional_drivers, or device_configurations.
+    if (loader_settings->debug_level != 0 || loader_settings->layer_configurations_active ||
+        loader_settings->additional_driver_count != 0 || loader_settings->device_configurations_active) {
+        loader_settings->settings_file_path = settings_file_path;
+        settings_file_path = NULL;
+        loader_settings->settings_active = true;
+    } else {
+        loader_log(inst, VULKAN_LOADER_INFO_BIT, 0,
+                   "vk_loader_settings.json file found at \"%s\" but did not contain any valid settings.", settings_file_path);
+    }
 out:
     if (NULL != json) {
         loader_cJSON_Delete(json);
@@ -545,7 +932,7 @@ TEST_FUNCTION_EXPORT VkResult get_settings_layers(const struct loader_instance*
 
     const loader_settings* settings = get_current_settings_and_lock(inst);
 
-    if (NULL == settings || !settings->settings_active) {
+    if (NULL == settings || !settings->settings_active || !settings->layer_configurations_active) {
         goto out;
     }
 
@@ -631,6 +1018,7 @@ TEST_FUNCTION_EXPORT VkResult get_settings_layers(const struct loader_instance*
             if (0 ==
                 strncmp(settings_layers->list[j].info.layerName, newly_added_layer->info.layerName, VK_MAX_EXTENSION_NAME_SIZE)) {
                 if (0 == (newly_added_layer->type_flags & VK_LAYER_TYPE_FLAG_META_LAYER) &&
+                    settings_layers->list[j].lib_name != NULL && newly_added_layer->lib_name != NULL &&
                     strcmp(settings_layers->list[j].lib_name, newly_added_layer->lib_name) == 0) {
                     should_remove = true;
                     break;
@@ -653,7 +1041,10 @@ out:
 // Skip comparing to UNORDERED_LAYER_LOCATION
 // If layer_property is a regular layer, check if the lib_path is the same.
 // Make sure that the lib_name pointers are non-null before calling strcmp.
-bool check_if_layer_is_in_list(struct loader_layer_list* layer_list, struct loader_layer_properties* layer_property) {
+// consider_lib_path - true if comparison should include the library_path in distinguishing layers. Used to prevent duplicates
+// between settings file provided layers and default found layers
+bool check_if_layer_is_in_list(struct loader_layer_list* layer_list, struct loader_layer_properties* layer_property,
+                               bool consider_lib_path) {
     // If the layer is a meta layer, just check against the name
     for (uint32_t i = 0; i < layer_list->count; i++) {
         if (0 == strncmp(layer_list->list[i].info.layerName, layer_property->info.layerName, VK_MAX_EXTENSION_NAME_SIZE)) {
@@ -663,6 +1054,9 @@ bool check_if_layer_is_in_list(struct loader_layer_list* layer_list, struct load
             if (VK_LAYER_TYPE_FLAG_META_LAYER == (layer_property->type_flags & VK_LAYER_TYPE_FLAG_META_LAYER)) {
                 return true;
             }
+            if (!consider_lib_path) {
+                return true;
+            }
             if (layer_list->list[i].lib_name && layer_property->lib_name) {
                 return strcmp(layer_list->list[i].lib_name, layer_property->lib_name) == 0;
             }
@@ -711,7 +1105,7 @@ VkResult combine_settings_layers_with_regular_layers(const struct loader_instanc
 
     // Insert the settings layers into output_layers up to unordered_layer_index
     for (uint32_t i = 0; i < unordered_layer_location_index; i++) {
-        if (!check_if_layer_is_in_list(output_layers, &settings_layers->list[i])) {
+        if (!check_if_layer_is_in_list(output_layers, &settings_layers->list[i], true)) {
             res = loader_append_layer_property(inst, output_layers, &settings_layers->list[i]);
             if (VK_SUCCESS != res) {
                 goto out;
@@ -721,8 +1115,8 @@ VkResult combine_settings_layers_with_regular_layers(const struct loader_instanc
 
     for (uint32_t i = 0; i < regular_layers->count; i++) {
         // Check if its already been put in the output_layers list as well as the remaining settings_layers
-        bool regular_layer_is_ordered = check_if_layer_is_in_list(output_layers, &regular_layers->list[i]) ||
-                                        check_if_layer_is_in_list(settings_layers, &regular_layers->list[i]);
+        bool regular_layer_is_ordered = check_if_layer_is_in_list(output_layers, &regular_layers->list[i], false) ||
+                                        check_if_layer_is_in_list(settings_layers, &regular_layers->list[i], false);
         // If it isn't found, add it
         if (!regular_layer_is_ordered) {
             res = loader_append_layer_property(inst, output_layers, &regular_layers->list[i]);
@@ -738,9 +1132,11 @@ VkResult combine_settings_layers_with_regular_layers(const struct loader_instanc
     // Insert the rest of the settings layers into combined_layers from  unordered_layer_index to the end
     // start at one after the unordered_layer_index
     for (uint32_t i = unordered_layer_location_index + 1; i < settings_layers->count; i++) {
-        res = loader_append_layer_property(inst, output_layers, &settings_layers->list[i]);
-        if (VK_SUCCESS != res) {
-            goto out;
+        if (!check_if_layer_is_in_list(output_layers, &settings_layers->list[i], true)) {
+            res = loader_append_layer_property(inst, output_layers, &settings_layers->list[i]);
+            if (VK_SUCCESS != res) {
+                goto out;
+            }
         }
     }
 
@@ -865,3 +1261,39 @@ VkResult enable_correct_layers_from_settings(const struct loader_instance* inst,
 out:
     return res;
 }
+
+VkResult loader_settings_get_additional_driver_files(const struct loader_instance* inst, struct loader_string_list* out_files) {
+    VkResult res = VK_SUCCESS;
+
+    const loader_settings* settings = get_current_settings_and_lock(inst);
+
+    if (NULL == settings || !settings->settings_active) {
+        goto out;
+    }
+
+    if (settings->additional_drivers_use_exclusively) {
+        free_string_list(inst, out_files);
+    }
+
+    for (uint32_t i = 0; i < settings->additional_driver_count; i++) {
+        res = prepend_if_manifest_file(inst, settings->additional_drivers[i].path, out_files);
+    }
+
+out:
+    release_current_settings_lock(inst);
+    return res;
+}
+
+bool loader_settings_should_use_driver_environment_variables(const struct loader_instance* inst) {
+    bool should_use = true;
+    const loader_settings* settings = get_current_settings_and_lock(inst);
+    if (NULL == settings || !settings->settings_active) {
+        goto out;
+    }
+    if (settings->device_configurations_active) {
+        should_use = false;
+    }
+out:
+    release_current_settings_lock(inst);
+    return should_use;
+}
