@@ -27,7 +27,14 @@
 
 #include "test_layer.h"
 
-#include "vk_dispatch_table_helper.h"
+#include <assert.h>
+
+#include <vulkan/vk_layer.h>
+
+#include "util/test_defines.h"
+#include "util/equality_helpers.h"
+
+#include "generated/vk_dispatch_table_helper.h"
 
 // export the enumeration functions instance|device+layer|extension
 #if !defined(TEST_LAYER_EXPORT_ENUMERATE_FUNCTIONS)
@@ -243,7 +250,7 @@ VKAPI_ATTR VkResult VKAPI_CALL test_vkCreateInstance(const VkInstanceCreateInfo*
     VkLayerInstanceCreateInfo* chain_info = get_chain_info(pCreateInfo, VK_LAYER_LINK_INFO);
 
     PFN_vkGetInstanceProcAddr fpGetInstanceProcAddr = chain_info->u.pLayerInfo->pfnNextGetInstanceProcAddr;
-    PFN_vk_icdGetPhysicalDeviceProcAddr fpGetPhysicalDeviceProcAddr = chain_info->u.pLayerInfo->pfnNextGetPhysicalDeviceProcAddr;
+    PFN_GetPhysicalDeviceProcAddr fpGetPhysicalDeviceProcAddr = chain_info->u.pLayerInfo->pfnNextGetPhysicalDeviceProcAddr;
     PFN_vkCreateInstance fpCreateInstance = (PFN_vkCreateInstance)fpGetInstanceProcAddr(NULL, "vkCreateInstance");
     if (fpCreateInstance == NULL) {
         return VK_ERROR_INITIALIZATION_FAILED;
